rm(list=ls())
graphics.off()
source("TaxonInfluenceIndex.R")


####################################
## CONSTRUCT JACKKNIFE ALIGNMENTS ##
####################################

## Done with jackknifeAlignments. The input alignment should be
## in PHYLIP format (interleaed or sequential). Jackknife alignments
## can be written all in one file or one per file. The last option
## is recommended if you have access to a cluster as you can then
## parallelize the jackknife trees inferences. 

## Alignment file name (Placental Mammals from the paper)
align <- "Data/Mammals/mammals.phy"

## Create jackknife alignments (all in one file)
jackknifeAlignments(align, format = "interleaved",
                    multi = FALSE, suffix = "jackknife")

## OR ALTERNATIVELY
## Create jackknife alignments (one per file)
jackknifeAlignments(align, format = "interleaved",
                    multi = TRUE, suffix = "jackknife")

########################
##  RECONSTRUCT TREES ##
########################

## Done with PhyML, the alignments produced by jackknifeAlignments
## are in PHYLIP format and can be used directly with PhyML


#################################
## COMPUTE BS AND TII INDEXES  ##
#################################

## Done with TaxonInfluence and BranchSupport. Both functions require
## as input a tree phy of class "phylo" and a list phylist of jackknife
## trees. Both check whether phylist is the complete list of jackknife
## trees. The trees can be either rooted or unrooted and either
## binary or multifurcating.


## random rooted tree
ex.tree <-  rtree(10)
## List of jackknife trees, generated to be compatible with complete tree
ex.tree.list <- lapply(1:10,FUN=function(x)drop.tip(ex.tree,paste("t",x,sep=""))) 
TaxonInfluence(ex.tree,ex.tree.list) ## 0 for all taxa
BranchSupport(ex.tree,ex.tree.list) ## 1 for all nodes

## random unrooted tree
ex.tree <- rtree(10,rooted=FALSE)
## List of jackknife trees, generated to be compatible with complete tree
ex.tree.list <- lapply(1:10,FUN=function(x)drop.tip(ex.tree,paste("t",x,sep="")))
TaxonInfluence(ex.tree,ex.tree.list) ## 0 for all taxa 
BranchSupport(ex.tree,ex.tree.list) ## 1 for all nodes 

## Test of validness of TII and BS values
## random trees of given size
areValuesCorrect <- function(n, rooted = TRUE)
  {
    ex.tree <-  rtree(10, rooted = rooted)
    ex.tree.list <- lapply(1:10,FUN=function(x)drop.tip(ex.tree,paste("t",x,sep=""))) 
    tii <- TaxonInfluence(ex.tree,ex.tree.list) ## 0 for all taxa
    bs <- BranchSupport(ex.tree,ex.tree.list) ## 1 for all nodes
    return(c(sum(tii != 0), sum(bs != 1))) 
  }

## For rooted trees
x <- replicate(100, areValuesCorrect(10, TRUE)); rowSums(x) ## 0 0 
## For unrooted trees
x <- replicate(100, areValuesCorrect(10, FALSE)); rowSums(x) ## 0 0 


## Real example 1: Placental Mammals
## data set described in Mariadassou et al., Syst Biol, 2012, 61(2):337-345
phy <-read.tree(file="Data/Mammals/mammals.phy_phyml_tree.txt")
phylist <-read.tree(file="Data/Mammals/mammals.jackknife.phy_phyml_tree.txt")
## OR ALTERNATIVELY
names.phylist <- list.files(path = "Data/Mammals",
                            pattern = "jackknife.+.phy_phyml_tree.txt",
                            full.names = TRUE)
phylist <- vector("list", length = length(names.phylist))
for (i in seq_along(names.phylist)) {
  phylist[[i]] <- read.tree(file = names.phylist[i])
}

tii <- TaxonInfluence(phy, phylist) 
bs <- BranchSupport(phy, phylist)

## Plot complete tree with BS and TII indexes
ordered.phy <- ordering.tips(phy) ## order tip labels 
plot(ordered.phy,show.tip.label=TRUE, tip.color = "white")
## plot branch support near corresponding nodes 
nodelabels(round(100*bs),node=length(tii)+1:length(bs),
           bg = "lightgreen", cex = 0.7)
## add tii values to species names and plot them near corresponding tip
tiplabels(mapply(paste,names(tii)," (",tii,")",sep=""),rank(names(tii)),
          frame="n",bg="",adj=c(-0.05,0.5), cex = 0.7) 


## Real example 2: Reptiles
## data set described in Mariadassou et al., Syst Biol, 2012, 61(2):337-345
phy <-read.tree(file="Data/Reptiles/reptile.phy_phyml_tree.txt")
phylist <-read.tree(file="Data/Reptiles/reptile.phy_phyml_tree_jackknife.txt")

tii <- TaxonInfluence(phy, phylist) 
bs <- BranchSupport(phy, phylist)

## Replace GenBank numbers by latin binomial
ordered.phy <- ordering.tips(phy) ## order tip labels
correspondance <- read.csv2("Data/Reptiles/correspondance_table.csv",
                            header = FALSE, row.names = 1,
                            stringsAsFactors = FALSE)
ordered.phy$tip.label <- correspondance[ordered.phy$tip.label, 1]
names(tii) <- correspondance[names(tii), 1]

## Plot complete tree with BS and TII indexes
plot(ordered.phy,show.tip.label=TRUE, tip.color = "white")
## plot branch support near corresponding nodes 
nodelabels(round(100*bs),node=length(tii)+1:length(bs),
           bg = "lightgreen", cex = 1)
## add tii values to species names and plot them near corresponding tip
tiplabels(mapply(paste,names(tii)," (",tii,")",sep=""),rank(names(tii)),
          frame="n",bg="",adj=c(-0.05,0.5), cex = 1) 




