## R function for computing Taxon Influence Index, as presented in
## Mariadassou, M., Bar-Hen, A. and Kishino, H. (2012), Syst. Biol., 61:(2),337-345
## by Mahendra Mariadassou, INRA Jouy-en-Josas, France (mahendra.mariadassou@jouy.inra.fr)
## Last edited 9th September 2013.

##  Copyright (C) 2012  Mahendra Mariadassou

##  This program is free software: you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation, either version 3 of the License, or
##  any later version.

##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##  GNU General Public License <http://www.gnu.org/licenses/> for more details.

## These functions require the following packages (available from http://cran.r-project.org/):

require(ape)

## REQUIREMENTS
## phy is an object of class phylo representing the complete tree (with all species)
## phylist is a list of objects of class phylo, each representing a jackknife tree (missing only one species when compared to the tree stored in phy)

####################
### Internals
####################

## Recode a clade X = {x_1, ..., x_n} of jackknife or complete tree 
## as a split of jackknifed tree corresponding to taxa i.
## The split X|Y = x_1,x_2,...| y_1,y_2,... but we conserve 
## only either of X or Y which contains a reference element (here 1).
## The set {x_1, ..., x_n, y_1, ..., y_p} must be {1, ..., N}
## Special Treatment for trivial splits stastifying |X| = 1 |X U {i}| = N-1
## which are always present in jackknife tree
code.partition.1 <- function(x, i, N) {
  ## Args:
  ##  x:  = c(x_1, ..., x_n) a numeric vector with the elements of X
  ##  i: taxon from complete tree missing from jackknife tree
  ##  N: cardinal of the partitioned set
  if (length(x) == 1 | (!i%in%x & (length(x) == N-2)) ) {
    ## special treatment for trivial splits
    y <- seq(N)
  } else {
    if (1%in%x) {
      y <- x
    } else {
      y <- c(1:N)[-x]
    }
  }
  ## remove jackknifed taxa from split to obtain a split
  ## from the jackknife tree
  return(y[y != i])
}


## Change each clade to a uniquely specified character string
## Used to prevent vector to vector comparison
code.partition.2 <- function(x)
  {
    paste(x, collapse = '#')
  }


## match tip labels in jackknife trees to tip labels in complete trees
## e.g. tip labels are then between 1:67, with a gap corresponding to
## the missing species instead of 1:66. Converts then clade to the corresponding
## bipartition. 
shift <- function(x, i, N)
  {
    ## Args:
    ##  x: clade of jackknife tree
    ##  i: taxon from complete tree missing from jackknife tree
    ##  N: number of taxa in complete tree
    y <- x
    y[x >= i] <- x[x >=i]+1
    y <- code.partition.1(y, i, N)
    return(y)
  }

## List version of shift
lshift <- function(li,i, N)
  {
    lapply(li,function(x) shift(x, i, N))
  }

## renumbering tips in the inferred trees by alphabetical
## order to achieve coherent name labeling in all trees
ordering.tips <- function(x)
  {
    ilab <- rank(x$tip.label)
    n <- length(ilab)
    ie <- match(1:n, x$edge[,2])
    x$edge[ie,2] <- ilab
    x$tip.label <- sort(x$tip.label)
    x
  } 

speciesclades <- function(phy,phylist)
  {
    ## Args
    ##  phy: object of class phylo representing the complete tree (with all species)
    ##  phylist: list of objects of class phylo, each representing a jackknife tree
    ##           (missing only one species when compared to the tree "phy")
    comp.tree <- ordering.tips(phy)
    jack.tree <- lapply(phylist,ordering.tips)
    nb.trees <- length(jack.tree)
    nb.species <- length(comp.tree$tip.label) ## nb.trees
    missing.species.list <- lapply(jack.tree,function(x)which(!comp.tree$tip%in%x$tip))
    missing.species <- unlist(missing.species.list)
    ## check whether all jackknifes trees are present
    if (nb.trees != nb.species)
      stop("Number of jackknife trees do not match number of taxa")
    ## check whether all trees are jackknife trees    
    if (any(unlist(lapply(missing.species.list, length)) != 1))
      stop("Some trees are not jackknife trees.")
    ## check whether all jackknife trees are present  
    if (any(sort(missing.species) != seq(nb.species)))
      stop("Some jackknife trees are missings.")
    names.species <- comp.tree$tip[missing.species]
    ## split decomposition of comp.tree
    decompose <- prop.part(comp.tree)
    ## split decomposition of jackknife trees
    decompose.boot <- lapply(jack.tree,prop.part)
    dec.boot.split <- mapply(lshift, decompose.boot, missing.species,
                             nb.species, SIMPLIFY = FALSE)
    ## Branch x Taxa matrix 
    result<-matrix(0, ncol = nb.trees, nrow = length(decompose),
                   dimnames = list(c(),names.species))
    ## match splits of jackknife trees to split of complete tree
    for (i in 1:nb.trees) {
      id.i <- missing.species[i]
      ## father node of jackknifed species, clade induced by this node
      ## is absent by construction from jackknifed tree 
      anc.i <- comp.tree$edge[comp.tree$edge[, 2] == id.i, 1] - nb.species
      ## Change clades from complete tree 
      dec.split.jack <- lapply(decompose, function(x) code.partition.1(x[x!=id.i], id.i, nb.species))
      clade.comp <- unlist(lapply(dec.split.jack, code.partition.2))
      clade.jack <-unlist(lapply(dec.boot.split[[i]], code.partition.2))
      result[,i] <- clade.comp%in%clade.jack+0
      ## Set "Missing by construction" clade to 0
      result[anc.i, i] <- 0
    }
    res <- list()
    res$mat <- result
    res$tree <- comp.tree
    res$labels <- names.species
    return(res)
  }


### Compute the branch lengths of edges delimiting the clades
clades.branch.length <- function(phy)
  {
    decompose <- prop.part(phy)
    cbl <- vector()
    cbl2 <- vector()
    for (j in 1:length(decompose))
      {
        tmp <- getMRCA(phy,decompose[[j]])
        tmp.edge.anc <- match(tmp,phy$edge[,2])
        if (is.na(tmp.edge.anc))
          cbl[j] <- 0
        else cbl[j] <- phy$edge.length[tmp.edge.anc]
      }
    for (j in 1:length(phy$tip)) {
     anc <- phy$edge[match(j,phy$edge[,2]),1]
     edge.anc <- match(anc,phy$edge[,2])
     if (is.na(edge.anc))
          cbl2[j] <- 0
        else cbl2[j] <- phy$edge.length[edge.anc]
   }
    names(cbl2) <- phy$tip.label
    list(cbl,cbl2)
  } 


#################
#### Externals
#################

## Read alignment and store it in a matrix
readAlignment <- function(file, format = "interleaved")
  {
    ## Args
    ##  file: alignment file name. Alignment must be in PHYLIP format
    dimensions <- as.numeric(read.table(file, nrows=1))
    number.species <- dimensions[1] ## number of taxa
    number.char <- dimensions[2]    ## number of characters
    ## read interleaved PHYLIP alignment
    if (format == "interleaved") {
      temp1 <-  read.table(file, skip=1, row.names=1,
                           nrows = number.species,
                           blank.lines.skip=TRUE, as.is=TRUE)
      if (nchar(paste(temp1[1,],collapse=""))<number.char) {
        ## if yes, some sequence data has not been read yet
        temp2 <- read.table(file,skip=1+number.species,blank.lines.skip=1,fill=TRUE)
        mul <- dim(temp2)[1]/number.species-1
        for (i in 0:mul) temp1 <- cbind(temp1,temp2[i*number.species+(1:number.species),])
      }
      align <- apply(temp1,1,function(x)as.character(paste(x,collapse="")))
      align <- cbind(names(align), align)
    }
    if (format == "sequential") {
      align <- matrix("", nrow = number.species, ncol = 2)
      Data <- scan(file = file, what = character(), sep = "\n", quiet = TRUE, 
                  skip = 1)
      j <- 1
      for (i in 1:number.species) {
        cur.split <- unlist(strsplit(Data[j], " +"))
        align[i, 1] <- Seq.split[1]
        cur.seq <- collapse(cur.split[-1], collapse = "")
        j <- j+1
        while (nchar(cur.seq) < number.char) {
          cur.seq <- paste(cur.seq, gsub(" ", "", Data[j]), sep = "")
          j <- j+1
        }
        align[i, 2] <- cur.seq
      }
    } 
    return(align)
  }

## Create jackknife alignment from an alignment matrix
jackknifeAlignments <- function(file, format = "interleaved",
                                multi = FALSE, suffix = "jackknife")
  {
    ## Args
    ##  align: alignment matrix
    ##  multi: should all alignments be written in the same file
    ##  suffix: suffix used to name new alignments
    align <- readAlignment(file = file, format = format)
    number.species <- nrow(align) ## number of taxa
    number.char <- nchar(align[1, 2])    ## number of characters
    filename <- unlist(strsplit(file, ".", fixed = TRUE))
    filename <- paste(filename[-length(filename)], sep = ".")
    ## output file for jackknife alignments
    align.jack <- paste(filename, suffix, "phy", sep = ".")
    for (i in 1:number.species){
      print(paste("Writing alignment obtained after pruning species", align[i, 1]))
      if (multi) {
        align.jack <- paste(filename, suffix, align[i, 1], "phy", sep = ".")
      }
      temp<-align[-i, ]
      ## format taxa names (replace '.' and '-' by '_')
      temp[ ,1] <- gsub("[.-]", "_", format(temp[ ,1]))
      write.table(paste(number.species-1, number.char),
                  file = align.jack, quote = FALSE,
                  sep=" ", row.names = FALSE,
                  col.names = FALSE, append = ifelse(i == 1, FALSE, TRUE))
      write.table(temp, file = align.jack, append = TRUE,
                  quote = FALSE, sep = "   ",
                  row.names = FALSE, col.names = FALSE)
    }
  }




## Compute Taxon Influence Index of each species of (complete) tree "phy"
TaxonInfluence <- function(phy, phylist, dist="RF")
  {
    ## Args
    ##  phy: object of class phylo representing the complete tree (with all species)
    ##  phylist: list of objects of class phylo, each representing a jackknife tree
    ##           (missing only one species when compared to the tree "phy")
    ##  dist: Either of "RF" or "BDS", specifies the weighting of the discordant branches
    ##        1 for RF or actual branch length for BDS. 
    sc <- speciesclades(phy,phylist)
    if (dist=="RF")
      TII<-2*(apply(1-sc$mat,2,sum)-1)
    if (dist=="BDS") {
      cbl <- clades.branch.length(sc$tree)
      TII <- apply(cbl[[1]]*(1-sc$mat),2,sum)-cbl[[2]][sc$labels]
    }
    names(TII) <- sc$labels
    return(TII)
  }

## Compute Branch Support of each branch of (complete) tree "phy"
BranchSupport <- function(phy,phylist)
  {
    ## Args
    ##  phy: object of class phylo representing the complete tree (with all species)
    ##  phylist: list of objects of class phylo, each representing a jackknife tree
    ##           (missing only one species when compared to the tree "phy")
    sc <- speciesclades(phy,phylist)
    nb.trees <- length(sc$labels)
    tree <- sc$tree
    nb.taxa <- length(phy$tip)
    BS <- apply(sc$mat,1,sum)
    ## compute max support for each branch/node by counting the number
    ## of extant taxa directly descending from that node. 
    BS.max <- nb.trees - unlist(lapply(1:nrow(sc$mat),
                                       function(x)sum(tree$edge[grep(x+nb.taxa,tree$edge[,1]),2] <= nb.taxa)))
    BS.max[1] <- BS[1] ## complete taxa set, always present by construction
    return(BS/BS.max)
  }
